from __future__ import annotations

from .deu.GermanSTSBenchmarkSTS import *
from .eng.BiossesSTS import *
from .eng.SickrSTS import *
from .eng.STS12STS import *
from .eng.STS13STS import *
from .eng.STS14STS import *
from .eng.STS15STS import *
from .eng.STS16STS import *
from .eng.STSBenchmarkSTS import *
from .fao.FaroeseSTS import *
from .fas.FaMTEBSTS import *
from .fin.FinParaSTS import *
from .fra.SickFrSTS import *
from .jpn.JSICK import *
from .jpn.JSTS import *
from .kor.KlueSTS import *
from .kor.KorSTS import *
from .multilingual.IndicCrosslingualSTS import *
from .multilingual.SemRel24STS import *
from .multilingual.STS17CrosslingualSTS import *
from .multilingual.STS22CrosslingualSTS import *
from .multilingual.STSBenchmarkMultilingualSTS import *
from .pol.PolishSTS import *
from .por.Assin2STS import *
from .por.SickBrSTS import *
from .ron.RonSTS import *
from .rus.RUParaPhraserSTS import *
from .rus.RuSTSBenchmarkSTS import *
from .spa.STSES import *
from .zho.CMTEBSTS import *
