from __future__ import annotations

from .deu.BlurbsClusteringP2P import *
from .deu.BlurbsClusteringS2S import *
from .deu.TenKGnadClusteringP2P import *
from .deu.TenKGnadClusteringS2S import *
from .eng.ArxivClusteringP2P import *
from .eng.ArxivClusteringS2S import *
from .eng.ArXivHierarchicalClustering import *
from .eng.BigPatentClustering import *
from .eng.BiorxivClusteringP2P import *
from .eng.BiorxivClusteringS2S import *
from .eng.BuiltBenchClusteringP2P import *
from .eng.BuiltBenchClusteringS2S import *
from .eng.MedrxivClusteringP2P import *
from .eng.MedrxivClusteringS2S import *
from .eng.RedditClustering import *
from .eng.RedditClusteringP2P import *
from .eng.StackExchangeClustering import *
from .eng.StackExchangeClusteringP2P import *
from .eng.TwentyNewsgroupsClustering import *
from .eng.WikiCitiesClustering import *
from .eng.WikipediaChemistrySpecialtiesClustering import *
from .eng.WikipediaChemistryTopicsClustering import *
from .fas.FaMTEBClustering import *
from .fra.AlloProfClusteringP2P import *
from .fra.AlloProfClusteringS2S import *
from .fra.HALClusteringS2S import *
from .jpn.LivedoorNewsClustering import *
from .jpn.MewsC16JaClustering import *
from .multilingual.IndicReviewsClusteringP2P import *
from .multilingual.MasakhaNEWSClusteringP2P import *
from .multilingual.MasakhaNEWSClusteringS2S import *
from .multilingual.MLSUMClusteringP2P import *
from .multilingual.MLSUMClusteringS2S import *
from .multilingual.SIB200ClusteringS2S import *
from .multilingual.WikiClusteringP2P import *
from .nob.snl_clustering import *
from .nob.SNLHierarchicalClustering import *
from .nob.vg_clustering import *
from .nob.VGHierarchicalClustering import *
from .pol.PolishClustering import *
from .rom.RomaniBibleClustering import *
from .rus.GeoreviewClusteringP2P import *
from .rus.RuSciBenchGRNTIClusteringP2P import *
from .rus.RuSciBenchOECDClusteringP2P import *
from .spa.SpanishNewsClusteringP2P import *
from .swe.swedn_clustering import *
from .swe.SwednClustering import *
from .zho.CMTEBClustering import *
