from __future__ import annotations

from .eng.Birdsnap import *
from .eng.Caltech101 import *
from .eng.CIFAR import *
from .eng.CLEVR import *
from .eng.Country211 import *
from .eng.DTD import *
from .eng.EuroSAT import *
from .eng.FER2013 import *
from .eng.FGVCAircraft import *
from .eng.Food101 import *
from .eng.GTSRB import *
from .eng.Imagenet1k import *
from .eng.MNIST import *
from .eng.OxfordPets import *
from .eng.PatchCamelyon import *
from .eng.RenderedSST2 import *
from .eng.RESISC45 import *
from .eng.SciMMIR import *
from .eng.StanfordCars import *
from .eng.STL10 import *
from .eng.SUN397 import *
from .eng.UCF101 import *
