from __future__ import annotations

from ..evaluation.LangMapping import *
from .AbsTask import *
from .AbsTaskBitextMining import *
from .AbsTaskClassification import *
from .AbsTaskClustering import *
from .AbsTaskInstructionRetrieval import *
from .AbsTaskMultilabelClassification import *
from .AbsTaskPairClassification import *
from .AbsTaskReranking import *
from .AbsTaskRetrieval import *
from .AbsTaskSpeedTask import *
from .AbsTaskSTS import *
from .AbsTaskSummarization import *
from .Image.AbsTaskAny2AnyMultiChoice import *
from .Image.AbsTaskAny2AnyRetrieval import *
from .Image.AbsTaskAny2TextMultipleChoice import *
from .Image.AbsTaskImageClassification import *
from .Image.AbsTaskImageClustering import *
from .Image.AbsTaskImageMultilabelClassification import *
from .Image.AbsTaskImageTextPairClassification import *
from .Image.AbsTaskVisualSTS import *
from .Image.AbsTaskZeroshotClassification import *
from .MultilingualTask import *
